/*
 * Copyright (c) 2011 Imre Fazekas.
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * Neither the name of the Brillien nor the names of its
 * terms and concepts may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.kernel;

/**
 * Basic Exception class used in almost every "problematical" behavior in Brillien
 */
public class BrillienException extends Exception {
    
    /**
     * Error code of the exception
     */
    protected int           errorCode;

    /**
     * Optional attachement value
     */
    protected Object        value;

    {
        errorCode = -1;
    }
    
    public BrillienException(String s) {
        this( -1, s );
    }

    public BrillienException(int errorCode, String s) {
        this( errorCode, s, null );
    }

    public BrillienException(int errorCode, String s, Object value) {
        super(s);
        this.errorCode = errorCode;
        this.value = value;
    }

    public BrillienException(Throwable cause) {
        super( getCause( cause) );
        this.errorCode = getCause() instanceof BrillienException ? ((BrillienException)getCause()).getErrorCode() : -1;
    }

    public BrillienException(int errorCode, Throwable cause) {
        super( getCause( cause) );
        this.errorCode = errorCode;
    }

    public BrillienException(String message, Throwable cause) {
        super(message, getCause( cause ) );
        this.errorCode = getCause() instanceof BrillienException ? ((BrillienException)getCause()).getErrorCode() : -1;
    }

    public BrillienException(int errorCode, String message, Throwable cause) {
        this( errorCode, message, cause, null );
    }
    public BrillienException(int errorCode, String message, Throwable cause, Object value) {
        super(message, getCause( cause ) );
        this.errorCode = errorCode;
        this.value = value;
    }

    /**
     * Getter method for field errorCode
     */
    public int getErrorCode() {
        return errorCode;
    }

    /**
     * Setter method for field errorCode
     */
    public void setErrorCode(int errorCode) {
        this.errorCode = errorCode;
    }

    /**
     * Setter method for field value
     */
    public Object getValue() {
        return value;
    }

    /**
     * Setter method for field value
     */
    public void setValue(Object value) {
        this.value = value;
    }

    protected static Throwable getCause( Throwable cause ){
        return cause == null || cause.getCause() == null ? cause : cause.getCause();
    }

    @Override
    public String getMessage() {
        return ("Code:" + errorCode + " Message:" ) + super.getMessage() + (value == null ? "" : " Value:" + value );
    }
}
