/*
 * Copyright (c) 2011 Imre Fazekas.
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * Neither the name of the Brillien nor the names of its
 * terms and concepts may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.kernel.axiom.sso;

import com.vii.brillien.kernel.BrillienException;

import java.util.List;

/**
 * Common interface used by a incorporated SSO service to provide A&A functionality
 */
public interface SubjectService {

    /**
     * Authenticates the user with the given username and principal
     */
    Session     authenticate( String username, String principal ) throws BrillienException;

    /**
     * Logs out the user associated with the given session
     */
    void        logout( Session session ) throws BrillienException;

    /**
     * Tells whether the user associated with the given session is authenticated
     */
    boolean     isAuthenticated(Session session) throws BrillienException;

    /**
     * Tells whether the user associated with the given session has the given role
     */
    boolean     hasRole(Session session, String roleIdentifier) throws BrillienException;

    /**
     * Tells whether the user associated with the given session has a role from the given collection
     */
    boolean     hasARole(Session session, List<String> roleIdentifiers) throws BrillienException;

    /**
     * Tells whether the user associated with the given session has all the given roles
     */
    boolean     hasRoles(Session session, List<String> roleIdentifiers) throws BrillienException;


    /**
     * Tells whether the user associated with the given session has the given permission
     */
    boolean     hasPermission(Session session, String permission) throws BrillienException;

    /**
     * Tells whether the user associated with the given session has a permission from the given collection
     */
    boolean     hasAPermission(Session session, List<String> permissions) throws BrillienException;

    /**
     * Tells whether the user associated with the given session has all the given permissions
     */
    boolean     hasPermissions(Session session, List<String> permissions) throws BrillienException;

}

